@@ -57,10 +57,16 @@ pkg_load = function(..., error = TRUE, install = FALSE) {
 #'   \code{\link{.packages}(TRUE)} (this does not really load the package, so it
 #'   is less rigorous but on the other hand, it can keep the current R session
 #'   clean).
+#' @param new_session Whether to test if a package is loadable in a new R
+#'   session. Note that \code{new_session = TRUE} implies \code{strict = TRUE}.
 #' @rdname pkg_attach
 #' @export
-loadable = function(pkg, strict = TRUE) {
-  if (strict) requireNamespace(pkg, quietly = TRUE) else pkg %in% .packages(TRUE)
+loadable = function(pkg, strict = TRUE, new_session = FALSE) {
+  if (new_session) {
+    Rscript(c('-e', shQuote(sprintf('library("%s")', pkg))), stdout = FALSE, stderr = FALSE) == 0
+  } else {
+    if (strict) requireNamespace(pkg, quietly = TRUE) else pkg %in% .packages(TRUE)
+  }
 }
 
 #' @rdname pkg_attach
@@ -70,3 +76,38 @@ pkg_attach2 = function(...) pkg_attach(..., install = TRUE)
 #' @rdname pkg_attach
 #' @export
 pkg_load2 = function(...) pkg_load(..., install = TRUE)
+
+
+broken_packages = function(reinstall = TRUE) {
+  pkgs = unlist(plapply(.packages(TRUE), function(p) if (!loadable(p)) p))
+  if (reinstall) {
+    remove.packages(pkgs); pkg_install(pkgs)
+  } else pkgs
+}
+
+#' Install a source package from a directory
+#'
+#' Run \command{R CMD build} to build a tarball from a source directory, and run
+#' \command{R CMD INSTALL} to install it.
+#' @param src The package source directory.
+#' @param build Whether to build a tarball from the source directory. If
+#'   \code{FALSE}, run \command{R CMD INSTALL} on the directory directly (note
+#'   that vignettes will not be automatically built).
+#' @param build_opts The options for \command{R CMD build}.
+#' @param install_opts The options for \command{R CMD INSTALL}.
+#' @export
+#' @return Invisible status from \command{R CMD INSTALL}.
+install_dir = function(src, build = TRUE, build_opts = NULL, install_opts = NULL) {
+  desc = file.path(src, 'DESCRIPTION')
+  pv = read.dcf(desc, fields = c('Package', 'Version'))
+  # delete existing tarballs
+  unlink(sprintf('%s_*.tar.gz', pv[1, 1]))
+  pkg = if (build) {
+    Rcmd(c('build', build_opts, shQuote(src)))
+    sprintf('%s_%s.tar.gz', pv[1, 1], pv[1, 2])
+  } else src
+  res = Rcmd(c('INSTALL', install_opts, pkg))
+  if (build) unlink(pkg)
+  if (res != 0) stop('Failed to install the package ', pkg)
+  invisible(res)
+}
